#!BPY 

""" 
Name: 'ASCII Scene (.ase) v0.5b' 
Blender: 236 
Group: 'Export' 
Tip: 'ASCII Scene Export (*.ase)' 
""" 
__version__ = "0.5b3" 

import Blender, mod_meshtools as meshtools 
import time, os 
from Blender import NMesh, Material, Texture, Image, Scene 

print "---------Start of Export------------" 

def write(filename): 
   start = time.clock() 

   global exp_list, Tab, idnt, imgTable 

   exp_list =[] 
   Tab = "\t" 
   idnt = 0 
   imgTable = {} 

   world = Blender.World.Get() 
   worldAmb = world[0].getAmb() 
   worldHor = world[0].getHor() 

   file = open(filename, "w") 
   set_up(file, exp_list, imgTable) 
   write_header(file, filename, worldAmb, worldHor) 
   write_materials(file, exp_list, worldAmb) 
   write_mesh(file, exp_list, imgTable) 
   file.close() 
    
   Blender.Window.DrawProgressBar(1.0, "")    # clear progressbar 
   end = time.clock() 
   seconds = " in %.2f %s" % (end-start, "seconds") 
   message = "Successfully exported " + os.path.basename(filename) + seconds 
   meshtools.print_boxed(message) 


#============================================ 
#           Setup 
#============================================ 

def set_up(file, exp_list, imgTable): 
   print "Setup" 

   if Blender.Object.GetSelected(): 
      objects = Blender.Object.GetSelected() 
      set_lists(file, exp_list, objects, imgTable) 
   else: 
      if Blender.Object.Get(): 
         objects = Blender.Object.Get() 
         set_lists(file, exp_list, objects, imgTable) 


def set_lists(file, exp_list, objects, imgTable): 
   global mat_cnt 
   mat_cnt = 0 
   mat_ref = 0 

   #exp_list = [container1 = [[mesh], [materials] or [uvimages], [material_ref], [mat_type]],...] 

   for current_obj in objects: 
      container = [] 
      if current_obj.getType() == 'Mesh': 
         container.append(current_obj) 

         mesh_mats = current_obj.getMaterials(0) 
         mat_type = 0 

         if len(mesh_mats) is not 0: 
            mat_type = 1 
            container.append(mesh_mats) 
            container.append(mat_ref) 
            container.append(mat_type) 
            mat_ref+=1 
            mat_cnt+=1 
         else: 
            face_img = [] 
            img_idx = 0 
            mesh = current_obj.data 
            for current_face in mesh.faces: 
               if current_face.image: 
                  if current_face.image not in face_img: 
                     face_img.append(current_face.image) 
                     cur_img = current_face.image.getName() 
                     imgTable[cur_img] = img_idx 
                     img_idx += 1 

            if len(face_img) is not 0: 
               mat_type = 2 
               container.append(face_img) 
               container.append(mat_ref) 
               container.append(mat_type) 
               mat_ref+=1 
               mat_cnt+=1 

         exp_list.append(container) 



#============================================ 
#           Header/Scene 
#============================================ 

def write_header(file, filename, worldAmb, worldHor): 
   print "Write Header" 
   scn = Scene.GetCurrent() 
   context = scn.getRenderingContext() 

   file.write("*3DSMAX_ASCIIEXPORT   200\n") 
   file.write("*COMMENT \"Exported from Blender\"\n") 
   file.write("*SCENE {\n") 
   file.write("%s*SCENE_FILENAME %s\n" % (Tab, os.path.basename(filename))) 

   file.write("%s*SCENE_FIRSTFRAME %d\n" % (Tab,context.startFrame())) 
   file.write("%s*SCENE_LASTFRAME %d\n" % (Tab,context.endFrame())) 
   file.write("%s*SCENE_FRAMESPEED %d\n" % (Tab,context.framesPerSec())) 
   file.write("%s*SCENE_TICKSPERFRAME 160\n" % (Tab)) #Blender have no Ticks? 

   file.write("%s*SCENE_BACKGROUND_STATIC %.4f %.4f %.4f\n" % (Tab, worldHor[0], worldHor[1], worldHor[2])) 
   file.write("%s*SCENE_AMBIENT_STATIC %.4f %.4f %.4f\n" % (Tab, worldAmb[0], worldAmb[1], worldAmb[2])) 
   file.write("}\n") 


#============================================ 
#           Materials 
#============================================ 

def write_materials(file, exp_list, worldAmb): 
   print "Write Materials" 

   file.write("*MATERIAL_LIST {\n") 
   file.write("%s*MATERIAL_COUNT %s\n" % (Tab, mat_cnt)) 

   for current_container in exp_list: 
       
      if len(current_container) is 4: # Materials exist 

         if len(current_container[1]) is 1: # single mat 
            mat_class = 'Standard' 

            if current_container[3] is 1: # Blender material 
               mats = current_container[1] 
               material = mats[0] 
               mat_name = material.name 

               file.write("%s*MATERIAL %d {\n" % ((Tab), current_container[2])) 

               idnt = 2 
               mat_para(file, idnt, material, mat_name, mat_class, worldAmb) 
               mat_dummy(file, idnt) 
               mat_map(file, idnt, mat_name) 

               file.write("%s}\n" % (Tab)) 

            else: # UV fake mat 
               if current_container[3] is 2: 
                  uv_imgs = current_container[1] 
                  uv_image = uv_imgs[0] 
                  uv_name = uv_image.getName() 

                  file.write("%s*MATERIAL %d {\n" % ((Tab), current_container[2])) 

                  idnt = 2 
                  mat_uv_para(file, idnt, uv_image, uv_name, mat_class, worldAmb) 
                  mat_dummy(file, idnt) 
                  mat_uv_map(file, idnt, uv_image, uv_name) 

                  file.write("%s}\n" % (Tab)) 

         else: # multiple mat 
            if len(current_container[1]) >= 1: 
               mat_class = 'Multi/Sub-Object' 

               if current_container[3] is 1: # Blender materials 
                  mats = current_container[1] 
                  material = mats[0] 
                  mat_name = 'Multi # ' + material.name 
                  submat_no = len(mats) 
                  submat_cnt = 0 

                  idnt = 2 
                  file.write("%s*MATERIAL %d {\n" % ((Tab), current_container[2])) 

                  mat_para(file, idnt, material, mat_name, mat_class, worldAmb) 

                  file.write("%s*NUMSUBMTLS %d\n" % ((Tab*idnt), submat_no)) 

                  for current_mat in mats: 
                     material = current_mat 
                     mat_class = 'Standard' 
                     mat_name = material.name 

                     idnt = 2 
                     file.write("%s*SUBMATERIAL %d {\n" % ((Tab*idnt), submat_cnt)) 
                     submat_cnt += 1 

                     idnt = 3 
                     mat_para(file, idnt, material, mat_name, mat_class, worldAmb) 
                     mat_dummy(file, idnt) 
                     mat_map(file, idnt, mat_name) 

                     idnt = 2 
                     file.write("%s}\n" % (Tab*idnt)) 

                  file.write("%s}\n" % (Tab)) 

               else: # UV fake mats 
                  if current_container[3] is 2: 
                     uv_imgs = current_container[1] 
                     uv_image = uv_imgs[0] 
                     uv_name = 'Multi # ' + uv_image.getName() 
                     submat_no = len(uv_imgs) 
                     submat_cnt = 0 

                     idnt = 2 
                     file.write("%s*MATERIAL %d {\n" % ((Tab), current_container[2])) 

                     mat_uv_para(file, idnt, uv_image, uv_name, mat_class, worldAmb) 

                     file.write("%s*NUMSUBMTLS %d\n" % ((Tab*idnt), submat_no)) 

                     for current_uv in uv_imgs: 
                        uv_image = current_uv 
                        mat_class = 'Standard' 
                        uv_name = uv_image.getName() 

                        idnt = 2 
                        file.write("%s*SUBMATERIAL %d {\n" % ((Tab*idnt), submat_cnt)) 
                        submat_cnt += 1 

                        idnt = 3 
                        mat_uv_para(file, idnt, uv_image, uv_name, mat_class, worldAmb) 
                        mat_dummy(file, idnt) 
                        mat_uv_map(file, idnt, uv_image, uv_name) 

                        idnt = 2 
                        file.write("%s}\n" % (Tab*idnt)) 

                     file.write("%s}\n" % (Tab)) 


   file.write("}\n") 


def mat_para(file, idnt, material, mat_name, mat_class, worldAmb): 

   mat_amb = material.getAmb() 
   mat_dif = material.getRGBCol() 
   mat_specCol = material.getSpecCol() 
   mat_spec = material.getSpec() 
   mat_hard = material.getHardness() 
   mat_alpha = 1.0000-material.getAlpha() 

   file.write("%s*MATERIAL_NAME \"%s\"\n" % ((Tab*idnt), mat_name)) 
   file.write("%s*MATERIAL_CLASS \"%s\"\n" % ((Tab*idnt), mat_class)) 
   file.write("%s*MATERIAL_AMBIENT %.4f   %.4f   %.4f\n" % ((Tab*idnt), (worldAmb[0]*mat_amb), (worldAmb[1]*mat_amb), (worldAmb[2]*mat_amb))) #-Usefull? 
   file.write("%s*MATERIAL_DIFFUSE %.4f   %.4f   %.4f\n" % ((Tab*idnt), mat_dif[0], mat_dif[1], mat_dif[2])) 
   file.write("%s*MATERIAL_SPECULAR %.4f   %.4f   %.4f\n" % ((Tab*idnt), mat_specCol[0], mat_specCol[1], mat_specCol[2])) 
   file.write("%s*MATERIAL_SHINE %.4f\n" % ((Tab*idnt), mat_spec)) 
   file.write("%s*MATERIAL_SHINESTRENGTH %.4f\n" % ((Tab*idnt), (mat_hard/511.))) #-511 or 512? 
   file.write("%s*MATERIAL_TRANSPARENCY %.4f\n" % ((Tab*idnt), mat_alpha)) 
   file.write("%s*MATERIAL_WIRESIZE 1.0000\n" % (Tab*idnt)) 


def mat_dummy(file, idnt): 

   file.write("%s*MATERIAL_SHADING Blinn\n" % (Tab*idnt)) 
   file.write("%s*MATERIAL_XP_FALLOFF 0.0000\n" % (Tab*idnt)) 
   file.write("%s*MATERIAL_SELFILLUM 0.0000\n" % (Tab*idnt)) 
   file.write("%s*MATERIAL_FALLOFF In\n" % (Tab*idnt)) 
   file.write("%s*MATERIAL_XP_TYPE Filter\n" % (Tab*idnt)) 


def mat_map(file, idnt, mat_name): 

      tex_list = [] 
      mat = Material.Get(mat_name) 
      MTexes = mat.getTextures() 

      # container = [[[MTex], [map_type/mapTo], [MAP_SUBNO]], ...] 

      # MAP_SUBNO 1 = MAP_DIFFUSE = COL = 1 
      for current_MTex in MTexes: 
         if current_MTex is not None: 

            container = [] 
            if current_MTex.mapto >= Texture.MapTo.COL: 
               map_type = '*MAP_DIFFUSE' 
               map_subNo = '1' 
               container.append(current_MTex) 
               container.append(map_type) 
               container.append(map_subNo) 
               tex_list.append(container) 


      # MAP_SUBNO 2 = MAP_SPECULAR = CSP or SPEC?= 4 
      for current_MTex in MTexes: 
         if current_MTex is not None: 
            container = [] 
            if current_MTex.mapto >= Texture.MapTo.CSP: 
               map_type = '*MAP_SPECULAR' 
               map_subNo = '2' 
               container.append(current_MTex) 
               container.append(map_type) 
               container.append(map_subNo) 
               tex_list.append(container) 

      # MAP_SUBNO 3 = MAP_SHINE = SPEC or CSP?= 32 
      for current_MTex in MTexes: 
         if current_MTex is not None: 
            container = [] 
            if current_MTex.mapto >= Texture.MapTo.SPEC: 
               map_type = '*MAP_SHINE' 
               map_subNo = '3' 
               container.append(current_MTex) 
               container.append(map_type) 
               container.append(map_subNo) 
               tex_list.append(container) 

      # MAP_SUBNO 4 = MAP_SHINESTRENGTH = HARD = 256 
      for current_MTex in MTexes: 
         if current_MTex is not None: 
            container = [] 
            if current_MTex.mapto >= Texture.MapTo.HARD: 
               map_type = '*MAP_SHINESTRENGTH' 
               map_subNo = '4' 
               container.append(current_MTex) 
               container.append(map_type) 
               container.append(map_subNo) 
               tex_list.append(container) 

      # MAP_SUBNO 5 = MAP_TRANSPARENCY = ALPHA = 128 
      for current_MTex in MTexes: 
         if current_MTex is not None: 
            container = [] 
            if current_MTex.mapto >= Texture.MapTo.ALPHA: 
               map_type = '*MAP_TRANSPARENCY' 
               map_subNo = '5' 
               container.append(current_MTex) 
               container.append(map_type) 
               container.append(map_subNo) 
               tex_list.append(container) 


      # MAP_SUBNO 8 = MAP_BUMP = NOR = 2  !Normalmap 2 Bumpmap! 
      for current_MTex in MTexes: 
         if current_MTex is not None: 
            container = [] 
            if current_MTex.mapto >= Texture.MapTo.NOR: 
               map_type = '*MAP_BUMP' 
               map_subNo = '8' 
               container.append(current_MTex) 
               container.append(map_type) 
               container.append(map_subNo) 
               tex_list.append(container) 



      for current_MTex in tex_list: 
         tex = current_MTex[0].tex 

         if tex.type == Texture.Types.IMAGE: 
            img = tex.getImage() 
            tex_class = 'Bitmap' 
            tex_mapType = 'Screen' 
            tex_filter = 'Pyramidal' 

            file.write("%s%s {\n" % ((Tab*idnt), current_MTex[1])) 

            idnt += 1 
            file.write("%s*MAP_NAME \"%s\"\n" % ((Tab*idnt), tex.getName())) 
            file.write("%s*MAP_CLASS \"%s\"\n" % ((Tab*idnt), tex_class)) 
            file.write("%s*MAP_SUBNO %s\n" % ((Tab*idnt), current_MTex[2])) 
            file.write("%s*MAP_AMOUNT 1.0000\n" % (Tab*idnt)) #-dont know how to get this value! 
            file.write("%s*BITMAP \"%s\"\n" % ((Tab*idnt), img.getFilename())) 
            file.write("%s*MAP_TYPE %s\n" % ((Tab*idnt), tex_mapType)) 

            # hope this part is right! 
            u_tiling = tex.repeat[0]*tex.crop[2] 
            v_tiling = tex.repeat[1]*tex.crop[3] 
            file.write("%s*UVW_U_OFFSET %.4f\n" % ((Tab*idnt), tex.crop[0])) 
            file.write("%s*UVW_V_OFFSET %.4f\n" % ((Tab*idnt), tex.crop[1])) 
            file.write("%s*UVW_U_TILING %.4f\n" % ((Tab*idnt), u_tiling)) 
            file.write("%s*UVW_V_TILING %.4f\n" % ((Tab*idnt), v_tiling)) 

            file.write("%s*UVW_ANGLE 0.0000\n" % (Tab*idnt)) 
            file.write("%s*UVW_BLUR 1.0000\n" % (Tab*idnt)) 
            file.write("%s*UVW_BLUR_OFFSET 0.0000\n" % (Tab*idnt)) 
            file.write("%s*UVW_NOUSE_AMT 1.0000\n" % (Tab*idnt)) 
            file.write("%s*UVW_NOISE_SIZE 1.0000\n" % (Tab*idnt)) 
            file.write("%s*UVW_NOISE_LEVEL 1\n" % (Tab*idnt)) 
            file.write("%s*UVW_NOISE_PHASE 0.0000\n" % (Tab*idnt)) 

            file.write("%s*BITMAP_FILTER %s\n" % ((Tab*idnt), tex_filter)) 

            idnt -= 1 
            file.write("%s}\n" % (Tab*idnt)) 


def mat_uv_para(file, idnt, uv_image, uv_name, mat_class, worldAmb): 
   fake_val0 = '0.0000' 
   fake_val1 = '0.1000' 
   fake_val2 = '0.5882' 
   fake_val3 = '0.9000' 
   fake_val4 = '1.0000' 


   file.write("%s*MATERIAL_NAME \"%s\"\n" % ((Tab*idnt), uv_name)) 
   file.write("%s*MATERIAL_CLASS \"%s\"\n" % ((Tab*idnt), mat_class)) 
   file.write("%s*MATERIAL_AMBIENT %.4f   %.4f   %.4f\n" % ((Tab*idnt), worldAmb[0], worldAmb[1], worldAmb[2])) #------------Usefull? 
   file.write("%s*MATERIAL_DIFFUSE %s   %s   %s\n" % ((Tab*idnt), fake_val2, fake_val2, fake_val2)) 
   file.write("%s*MATERIAL_SPECULAR %s   %s   %s\n" % ((Tab*idnt), fake_val3, fake_val3, fake_val3)) 
   file.write("%s*MATERIAL_SHINE %s\n" % ((Tab*idnt), fake_val1)) 
   file.write("%s*MATERIAL_SHINESTRENGTH %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*MATERIAL_TRANSPARENCY %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*MATERIAL_WIRESIZE %s\n" % ((Tab*idnt), fake_val4)) 


def mat_uv_map(file, idnt, uv_image, uv_name): 
   map_type = '*MAP_DIFFUSE' 
   map_subNo = '1' 
   tex_class = 'Bitmap' 
   tex_mapType = 'Screen' 
   tex_filter = 'Pyramidal' 

   fake_val0 = '0.0000' 
   fake_val1 = '0.1000' 
   fake_val2 = '0.5882' 
   fake_val3 = '0.9000' 
   fake_val4 = '1.0000' 

   #replace "/" with "\" in path 
   uv_filename = uv_image.getFilename().replace('/', '\\') 

   file.write("%s%s {\n" % ((Tab*idnt), map_type)) 

   idnt += 1 
   file.write("%s*MAP_NAME \"%s\"\n" % ((Tab*idnt), uv_name)) 
   file.write("%s*MAP_CLASS \"%s\"\n" % ((Tab*idnt), tex_class)) 
   file.write("%s*MAP_SUBNO %s\n" % ((Tab*idnt), map_subNo)) 
   file.write("%s*MAP_AMOUNT %s\n" % ((Tab*idnt), fake_val4)) 
   file.write("%s*BITMAP \"%s\"\n" % ((Tab*idnt), uv_filename)) 
   file.write("%s*MAP_TYPE %s\n" % ((Tab*idnt), tex_mapType)) 
   file.write("%s*UVW_U_OFFSET %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*UVW_V_OFFSET %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*UVW_U_TILING %s\n" % ((Tab*idnt), fake_val4)) 
   file.write("%s*UVW_V_TILING %s\n" % ((Tab*idnt), fake_val4)) 
   file.write("%s*UVW_ANGLE %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*UVW_BLUR %s\n" % ((Tab*idnt), fake_val4)) 
   file.write("%s*UVW_BLUR_OFFSET %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*UVW_NOUSE_AMT %s\n" % ((Tab*idnt), fake_val4)) 
   file.write("%s*UVW_NOISE_SIZE %s\n" % ((Tab*idnt), fake_val4)) 
   file.write("%s*UVW_NOISE_LEVEL 1\n" % (Tab*idnt)) 
   file.write("%s*UVW_NOISE_PHASE %s\n" % ((Tab*idnt), fake_val0)) 
   file.write("%s*BITMAP_FILTER %s\n" % ((Tab*idnt), tex_filter)) 

   idnt -= 1 
   file.write("%s}\n" % (Tab*idnt)) 







#============================================ 
#           Mesh 
#============================================ 

def write_mesh(file, exp_list, imgTable): 
   print "Write Geometric" 


   for current_container in exp_list: 
      object = current_container[0].data 
      vertices = object.verts 
      faces = object.faces 
      euler = current_container[0].getEuler() 
      location = current_container[0].getLocation() 
      scale = current_container[0].getSize() 

   #-------------------geom_header----------- 

      file.write("*GEOMOBJECT {\n") 
      file.write("%s*NODE_NAME \"%s\"\n" % (Tab, current_container[0].name)) 
      file.write("%s*NODE_TM {\n" % (Tab)) 

      idnt = 2 
      file.write("%s*NODE_NAME \"%s\"\n" % ((Tab*idnt), current_container[0].name)) 
      file.write("%s*INHERIT_POS 0 0 0\n" % (Tab*idnt)) 
      file.write("%s*INHERIT_ROT 0 0 0\n" % (Tab*idnt)) 
      file.write("%s*INHERIT_SCL 0 0 0\n" % (Tab*idnt)) 

      file.write("%s*TM_ROW0 1.0000 0.0000 0.0000\n" % (Tab*idnt)) 
      file.write("%s*TM_ROW1 0.0000 1.0000 0.0000\n" % (Tab*idnt)) 
      file.write("%s*TM_ROW2 0.0000 0.0000 1.0000\n" % (Tab*idnt)) 

      file.write("%s*TM_ROW3 %.4f %.4f %.4f\n" % ((Tab*idnt), location[0], location[1], location[2])) 
      #   file.write("%s*TM_ROW3 0.0000 0.0000 0.0000\n" % (Tab*idnt)) 

      file.write("%s*TM_POS %.4f %.4f %.4f\n" % ((Tab*idnt), location[0], location[1], location[2])) 
      file.write("%s*TM_ROTAXIS %.4f %.4f %.4f\n" % ((Tab*idnt), euler[0], euler[1], euler[2])) 
      file.write("%s*TM_ROTANGLE 0.0000\n" % (Tab*idnt)) 
      file.write("%s*TM_SCALE %.4f %.4f %.4f\n" % ((Tab*idnt), scale[0], scale[1], scale[2])) 
      file.write("%s*TM_SCALEAXIS 0.0000 0.0000 0.0000\n" % (Tab*idnt)) 

      #   file.write("%s*TM_SCALEAXIS %.4f %.4f %.4f\n" % ((Tab*idnt), euler[0], euler[1], euler[2])) 
      file.write("%s*TM_SCALEAXISANG 0.0000\n" % (Tab*idnt)) 

      file.write("%s}\n" % (Tab)) 

   #------------------VertexList-------------- 
      file.write("%s*MESH {\n" % (Tab)) 

      idnt = 2 
      file.write("%s*TIMEVALUE 0\n" % (Tab*idnt)) 
      file.write("%s*MESH_NUMVERTEX %d\n" % ((Tab*idnt), len(vertices))) 

      face_cnt = 0 

      for current_face in faces: 
         if len(current_face.v) is 3: 
            face_cnt += 1 
         else: 
            if len(current_face.v) is 4: 
               face_cnt += 2 

      file.write("%s*MESH_NUMFACES %d\n" % ((Tab*idnt), face_cnt)) 

      file.write("%s*MESH_VERTEX_LIST {\n" % (Tab*idnt)) 

      idnt = 3 
      vert_idx = 0 
      vert_cnt = len(vertices) 
      vertTable = {} 

      Blender.Window.DrawProgressBar(0, "Writing Vertices") 

      for current_vert in vertices: 
         vertTable[current_vert] = vert_idx 

         if (vert_idx % 1000) == 0: 
                      Blender.Window.DrawProgressBar(float(vert_idx) / vert_cnt, "Writing vertices") 

         coord = current_vert.co 
         file.write("%s*MESH_VERTEX %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vert_idx, coord[0], coord[1], coord[2])) 
         vert_idx += 1 

      idnt = 2 
      file.write("%s}\n" % (Tab*idnt)) 

   #------------------FaceList---------------- 
      file.write("%s*MESH_FACE_LIST {\n" % (Tab*idnt)) 
      idnt = 3 
      faceNo = 0 

      Blender.Window.DrawProgressBar(0, "Writing faces") 

      for current_face in faces: 
         verts = current_face.v 
         smooth = '*MESH_SMOOTHING 0' 

         if (faceNo % 500) == 0: 
            Blender.Window.DrawProgressBar(float(faceNo) / len(faces), "Writing faces") 

         matID = 0 
         if len(current_container) is 4: # Materials exist? 
            if current_container[3] is 1: # Blender mats 
               matID = current_face.materialIndex 
             
            else: 
               if current_container[3] is 2: # UV fake mats 
                  cur_img = current_face.image.getName() 
                  matID = imgTable[cur_img] 

         if len(verts) is 3: 
            vertA = vertTable[verts[0]] 
            vertB = vertTable[verts[1]] 
            vertC = vertTable[verts[2]] 
            file.write("%s*MESH_FACE %4d:    A: %4d B: %4d C: %4d AB:    1 BC:    1 CA:    1 %s   *MESH_MTLID %s\n" % ((Tab*idnt), faceNo, vertA, vertB, vertC, smooth, matID)) 
            faceNo+=1 
         else: 
            if len(verts) is 4: 
               vertA = vertTable[verts[0]] 
               vertB = vertTable[verts[1]] 
               vertC = vertTable[verts[2]] 
               vertD = vertTable[verts[3]] 
               file.write("%s*MESH_FACE %4d:    A: %4d B: %4d C: %4d AB:    1 BC:    1 CA:    0 %s   *MESH_MTLID %s\n" % ((Tab*idnt), faceNo, vertA, vertB, vertC, smooth, matID)) 
               faceNo+=1 
               file.write("%s*MESH_FACE %4d:    A: %4d B: %4d C: %4d AB:    1 BC:    1 CA:    0 %s   *MESH_MTLID %s\n" % ((Tab*idnt), faceNo, vertC, vertD, vertA, smooth, matID)) 
               faceNo+=1 
      idnt = 2 
      file.write("%s}\n" % (Tab*idnt)) 

   #-------------------UvVertsIndex---------------- 
      has_UV = object.hasFaceUV() 

      if has_UV == True: 
         UV_list = [] 
         UV_idx = 0 
         UVTable = {} 

         Blender.Window.DrawProgressBar(0, "Writing UV vertices") 
         cfacetmp = 1 
         nfacetmp = len(faces) 

         for current_face in faces: 

            if (cfacetmp % 1000) == 0: 
               #Blender.Window.DrawProgressBar(float(cfacetmp) / nfacetmp, "Writing UV vertices") 
               Blender.Window.DrawProgressBar(float(cfacetmp) / face_cnt, "Writing UV vertices") 

            vertUV = current_face.uv 
            for current_uv in vertUV: 
               if not UVTable.has_key(current_uv): 
               #if current_uv not in UV_list: 
                  UV_list.append(current_uv) 
                  UVTable[current_uv] = UV_idx 
                  UV_idx += 1 
            cfacetmp += 1 

         file.write("%s*MESH_NUMTVERTEX %d\n" % ((Tab*idnt), len(UV_list))) 
         file.write("%s*MESH_TVERTLIST {\n" % (Tab*idnt)) 

         idnt = 3 
         for current_UV in UV_list: 
            cooU = current_UV[0] 
            cooV = current_UV[1] 
            file.write("%s*MESH_TVERT %d    %.4f    %.4f    0.0000\n" % ((Tab*idnt), UVTable[current_UV], cooU, cooV)) 

         idnt = 2 
         file.write("%s}\n" % (Tab*idnt)) 

   #------------------UVfacebinds------------------ 
      if has_UV == True: 

         Blender.Window.DrawProgressBar(0, "Writing UV binds") 

         idnt = 2 
         file.write("%s*MESH_NUMTVFACES %d\n" % ((Tab*idnt), face_cnt)) 
         file.write("%s*MESH_TFACELIST {\n" % (Tab*idnt)) 
       
         idnt = 3 
         tfaceNo = 0 
         for current_face in faces: 


            if (tfaceNo % 1000) == 0: 
               Blender.Window.DrawProgressBar(float(tfaceNo) / face_cnt, "Writing UV binds") 

            if len(current_face.uv) is 3: 
               vertUV = current_face.uv 
               UV0 = UVTable[vertUV[0]] 
               UV1 = UVTable[vertUV[1]] 
               UV2 = UVTable[vertUV[2]] 
               file.write("%s*MESH_TFACE %d   %d   %d   %d\n" % ((Tab*idnt), tfaceNo, UV0, UV1, UV2)) 
               tfaceNo+=1 
            else: 
                if len(current_face.uv) is 4: 
                  vertUV = current_face.uv 
                  UV0 = UVTable[vertUV[0]] 
                  UV1 = UVTable[vertUV[1]] 
                  UV2 = UVTable[vertUV[2]] 
                  UV3 = UVTable[vertUV[3]] 
                  file.write("%s*MESH_TFACE %d   %d   %d   %d\n" % ((Tab*idnt), tfaceNo, UV0, UV1, UV2)) 
                  tfaceNo+=1 
                  file.write("%s*MESH_TFACE %d   %d   %d   %d\n" % ((Tab*idnt), tfaceNo, UV2, UV3, UV0)) 
                  tfaceNo+=1 

         idnt = 2 
         file.write("%s}\n" % (Tab*idnt)) 

   #-----------------VertexColorIndex------------------- 

      has_cVert = 0 
      if object.hasVertexColours() is 1: 
         has_cVert = 1 
      else: 
         if len(current_container) is 4: # Materials exist? 

            if current_container[3] is 1: # Blender material 
               mats = current_container[1] 
               mat = mats[0] 

               if mat.getMode() & Material.Modes['VCOL_PAINT']: 
                  has_cVert = 1 

      if has_cVert is 1: 
         cVert_list = [] 
         cVertTable = {} 
         cVertNo = 0 

         for current_face in faces: 
            cVert_all = current_face.col 

            for current_col in cVert_all: 
               color = [current_col.r, current_col.g, current_col.b] 
               colorT = '%f   %f   %f' % (current_col.r, current_col.g, current_col.b) 

               if color not in cVert_list: 
                  cVert_list.append(color) 
                  cVertTable[colorT] = cVertNo 
                  cVertNo += 1 

            if len(cVert_list) is 0: 
               has_cVert = 0 

      if has_cVert is 1: 

         Blender.Window.DrawProgressBar(0, "Writing VCol index") 
         bar_cno = 0 

         idnt = 2                      
         file.write("%s*MESH_NUMCVERTEX %d\n" % ((Tab*idnt), cVertNo)) 
         file.write("%s*MESH_CVERTLIST {\n" % (Tab*idnt)) 

         idnt = 3 
         for current_cvert in cVert_list: 

            if (bar_cno % 1000) == 0: 
               Blender.Window.DrawProgressBar(float(bar_cno) / cVertNo, "Writing UV vertices") 

            color = '%f   %f   %f' % (current_cvert[0], current_cvert[1], current_cvert[2]) 

            file.write("%s*MESH_VERTCOL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), cVertTable[color], (current_cvert[0]/255.), (current_cvert[1]/255.), (current_cvert[2]/255.))) 

            bar_cno += 1 

         idnt = 2 
         file.write("%s}\n" % (Tab*idnt)) 

   #-----------------FaceColorlist------------------- 
      if has_cVert is 1: 
         idnt = 2 
         cFaceNo = 0 

         Blender.Window.DrawProgressBar(0, "Writing VCol binds") 

         file.write("%s*MESH_NUMCFACES %d\n" % ((Tab*idnt), face_cnt)) 
         file.write("%s*MESH_CFACELIST {\n" % (Tab*idnt)) 

         idnt = 3 
         for current_face in faces: 

            if (cFaceNo % 500) == 0: 
               Blender.Window.DrawProgressBar(float(cFaceNo) / face_cnt, "Writing VCol binds") 

            cVertices = current_face.col 

            if len(current_face.v) is 3: 
               color0 = '%f   %f   %f' % (cVertices[0].r, cVertices[0].g, cVertices[0].b) 
               color1 = '%f   %f   %f' % (cVertices[1].r, cVertices[1].g, cVertices[1].b) 
               color2 = '%f   %f   %f' % (cVertices[2].r, cVertices[2].g, cVertices[2].b) 

               cVert0 = cVertTable[color0] 
               cVert1 = cVertTable[color1] 
               cVert2 = cVertTable[color2] 

               file.write("%s*MESH_CFACE %d   %d   %d   %d\n" % ((Tab*idnt), cFaceNo, cVert0, cVert1, cVert2)) 
               cFaceNo+= 1 
            else: 
               if len(current_face.v) is 4: 
                  color0 = '%f   %f   %f' % (cVertices[0].r, cVertices[0].g, cVertices[0].b) 
                  color1 = '%f   %f   %f' % (cVertices[1].r, cVertices[1].g, cVertices[1].b) 
                  color2 = '%f   %f   %f' % (cVertices[2].r, cVertices[2].g, cVertices[2].b) 
                  color3 = '%f   %f   %f' % (cVertices[3].r, cVertices[3].g, cVertices[3].b) 

                  cVert0 = cVertTable[color0] 
                  cVert1 = cVertTable[color1] 
                  cVert2 = cVertTable[color2] 
                  cVert3 = cVertTable[color3] 

                  file.write("%s*MESH_CFACE %d   %d   %d   %d\n" % ((Tab*idnt), cFaceNo, cVert0, cVert1, cVert2)) 
                  cFaceNo+= 1 
                  file.write("%s*MESH_CFACE %d   %d   %d   %d\n" % ((Tab*idnt), cFaceNo, cVert2, cVert3, cVert0)) 
                  cFaceNo+= 1 

         idnt = 2 
         file.write("%s}\n" % (Tab*idnt)) 

   #-----------------Normallist-------------------- 
      idnt = 2 
      file.write("%s*MESH_NORMALS {\n" % (Tab*idnt)) 

      idnt = 3 
      normNo = 0 
      for current_face in faces: 

         if (normNo % 500) == 0: 
            Blender.Window.DrawProgressBar(float(normNo) / face_cnt, "Writing Normals") 

         if len(current_face.v) is 3: 
            idnt = 3 
            face_norm = current_face.normal 
            file.write("%s*MESH_FACENORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), normNo, face_norm[0], face_norm[1], face_norm[2])) 
            normNo+=1 

            face_verts = current_face.v 
            vert0 = face_verts[0] 
            vert1 = face_verts[1] 
            vert2 = face_verts[2] 
            vert_norm0 = vert0.no 
            vert_norm1 = vert1.no 
            vert_norm2 = vert2.no 
            idnt = 4 
            file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[0]], vert_norm0[0], vert_norm0[1], vert_norm0[2])) 
            file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[1]], vert_norm1[0], vert_norm1[1], vert_norm1[2])) 
            file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[2]], vert_norm2[0], vert_norm2[1], vert_norm2[2])) 
         else: 
             if len(current_face.v) is 4: 
               face_verts = current_face.v 
               face_norm = current_face.normal 
               vert0 = face_verts[0] 
               vert1 = face_verts[1] 
               vert2 = face_verts[2] 
               vert3 = face_verts[3] 
               vert_norm0 = vert0.no 
               vert_norm1 = vert1.no 
               vert_norm2 = vert2.no 
               vert_norm3 = vert3.no 

               idnt = 3 
               file.write("%s*MESH_FACENORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), normNo, face_norm[0], face_norm[1], face_norm[2])) 
               normNo+=1 
               idnt = 4 
               file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[0]], vert_norm0[0], vert_norm0[0], vert_norm0[2])) 
               file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[1]], vert_norm1[0], vert_norm1[1], vert_norm1[2])) 
               file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[2]], vert_norm2[0], vert_norm2[1], vert_norm2[2])) 

               idnt = 3 
               file.write("%s*MESH_FACENORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), normNo, face_norm[0], face_norm[1], face_norm[2])) 
               normNo+=1 
               idnt = 4 
               file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[2]], vert_norm2[0], vert_norm2[1], vert_norm2[2])) 
               file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[3]], vert_norm3[0], vert_norm3[1], vert_norm3[2])) 
               file.write("%s*MESH_VERTEXNORMAL %d   %.4f   %.4f   %.4f\n" % ((Tab*idnt), vertTable[face_verts[0]], vert_norm0[0], vert_norm0[1], vert_norm0[2])) 
                

      idnt = 2 
      file.write("%s}\n" % (Tab*idnt)) 

      file.write("%s}\n" % (Tab)) 

   #--------------------Geom_Footer-------------------- 

      file.write("%s*PROP_MOTIONBLUR 0\n" % (Tab)) 
      file.write("%s*PROP_CASTSHADOW 1\n" % (Tab)) 
      file.write("%s*PROP_RECVSHADOW 1\n" % (Tab)) 

      if len(current_container) is not 1: 
         file.write("%s*MATERIAL_REF %d\n" % (Tab, current_container[2])) 

      file.write("}\n") 


   #-------------------------End?---------------------- 


def fs_callback(filename): 
   if filename.find('.ase', -4) <= 0: filename += '.ase' 
   write(filename) 

Blender.Window.FileSelector(fs_callback, "Export ASE")